uniform sampler2D shadowMap;
uniform sampler2D normal2DTex;
uniform sampler3D normal3DTex;

uniform int activeTexture;
uniform int useShadows;
uniform int useShading;
uniform float darking;

varying vec4 diffuse, ambient;
varying vec3 normal;
varying vec4 lightVec;

const vec4 AMBIENT_BLACK = vec4 (0.0, 0.0, 0.0, 1.0);
const vec4 DEFAULT_BLACK = vec4 (0.0, 0.0, 0.0, 0.0);

float calcShadow(float s, float t, vec4 shadowCoordProject)
{
	if (texture2D(shadowMap, vec2(s,t)).z < shadowCoordProject.z) {
			return 0.3;
		} else {
			return 0.5;
		}
}

bool isLightEnabled(){
	bool enabled = true;
	if((gl_LightSource[1].ambient == AMBIENT_BLACK) && 
			(gl_LightSource[1].diffuse == DEFAULT_BLACK) && 
				(gl_LightSource[1].specular == DEFAULT_BLACK)){
		enabled = false;			
	}
	return enabled;
}

void main()
{

	float shadow = 1.0;

	if (useShadows == 1) {
		vec4 shadowCoordProject = gl_TexCoord[7]/gl_TexCoord[7].w;
		//shadowCoordProject.z += 0.00002;
		
		float s = shadowCoordProject.s;
		float t = shadowCoordProject.t;
		
		
		float step = 0.0001;
		
		float distanceFromLight = texture2D(shadowMap, shadowCoordProject.st).z;
		
		if (gl_TexCoord[7].w > 0.0) {
			//shadow = calcShadow(s, t, shadowCoordProject);
			shadow += 25.0*calcShadow(s, t, shadowCoordProject);
			shadow += 15.0*calcShadow(s+step, t, shadowCoordProject);
			shadow += 15.0*calcShadow(s, t+step, shadowCoordProject);
			shadow += 15.0*calcShadow(s-step, t, shadowCoordProject);
			shadow += 15.0*calcShadow(s, t-step, shadowCoordProject);
			shadow += 9.0*calcShadow(s+step, t+step, shadowCoordProject);
			shadow += 9.0*calcShadow(s-step, t+step, shadowCoordProject);
			shadow += 9.0*calcShadow(s-step, t-step, shadowCoordProject);
			shadow += 9.0*calcShadow(s+step, t-step, shadowCoordProject);
			shadow += 5.0*calcShadow(s, t+step*2, shadowCoordProject);
			shadow += 5.0*calcShadow(s+step*2, t, shadowCoordProject);
			shadow += 5.0*calcShadow(s, t-step*2, shadowCoordProject);
			shadow += 5.0*calcShadow(s-step*2, t, shadowCoordProject);
			shadow += 3.0*calcShadow(s-step*2, t-step, shadowCoordProject);
			shadow += 3.0*calcShadow(s-step, t-step*2, shadowCoordProject);
			shadow += 3.0*calcShadow(s-step*2, t+step, shadowCoordProject);
			shadow += 3.0*calcShadow(s-step, t+step*2, shadowCoordProject);
			shadow += 3.0*calcShadow(s+step*2, t-step, shadowCoordProject);
			shadow += 3.0*calcShadow(s+step, t-step*2, shadowCoordProject);
			shadow += 3.0*calcShadow(s+step*2, t+step, shadowCoordProject);
			shadow += 3.0*calcShadow(s+step, t+step*2, shadowCoordProject);
			shadow += 1.0*calcShadow(s+step*2, t-step*2, shadowCoordProject);
			shadow += 1.0*calcShadow(s-step*2, t-step*2, shadowCoordProject);
			shadow += 1.0*calcShadow(s+step*2, t+step*2, shadowCoordProject);
			shadow += 1.0*calcShadow(s-step*2, t+step*2, shadowCoordProject);
			shadow /= 169.0;
		}else {
			shadow = 1.0;
		}
	}
	
	vec4 color;
	
	
	vec4 light = vec4(0.1);
	
	if (isLightEnabled()){
		
		vec4 ambient = vec4(0.0);
		vec4 diffuse = vec4(0.0);
		vec4 specular = vec4(0.0);
		
		vec3 N = normalize(normal);
	
		vec3 L = normalize(gl_LightSource[1].position.xyz);
		
		float NdotL = dot(N, L);

		ambient = gl_FrontMaterial.ambient * (gl_LightSource[1].ambient + gl_LightModel.ambient);
		
		light += ambient;
		
		if (NdotL > 0.0){
			
			vec3 H = gl_LightSource[1].halfVector.xyz;
		
			specular = gl_FrontMaterial.specular * gl_LightSource[1].specular * pow(max(dot(N, H), 0.0), gl_FrontMaterial.shininess);
			diffuse = gl_FrontMaterial.diffuse * gl_LightSource[1].diffuse * max(0.0,min(NdotL,1.0));
			
			light += specular;
			light += diffuse;
			
		}
		
		if (NdotL < 0.01){
			//shadow = min(1.0,-NdotL/10);
		}
		
		///gl_FragColor = max(0.0,min(NdotL,1.0));
		//gl_FragColor = color * light;
		
		
	}
	
	shadow = min(1.0,max(shadow,0.0));
	darking = min(1.0,max(darking,0.0));
	
	if (activeTexture == 2){
		color = darking * gl_Color * shadow * texture2D(normal2DTex, gl_TexCoord[2].st);
	}else if (activeTexture == 3){
		color = darking * gl_Color * shadow * texture3D(normal3DTex, gl_TexCoord[3].xyz);
	}else {
		color = gl_Color;
	}
	
	
	gl_FragColor = color * vec4(vec3(light),0);
	
}

